# -*- coding: utf-8 -*-
# Copyright © 2011 Nicolas Paris <nicolas.caen@gmail.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
import os
import sys
import time
import tyrs
import signal                   # resize event
import curses
import logging
from user import User
from utils import html_unescape, encode, get_source, get_urls

class Interface(object):
    ''' All dispositions in the screen

    self.api              The tweetter API (not directly the api, but the instance of Tweets in tweets.py)
    self.conf             The configuration file parsed in config.py
    self.maxyx            Array contain the window size [y, x]
    self.screen           Main screen (curse)
    self.current_y        Current line in the screen
    self.resize_event     boleen if the window is resize
    self.regexRetweet     regex for retweet
    self.refresh_token    Boleen to make sure we don't refresh timeline. Usefull to keep editing box on top
    self.buffer           The current buffer we're looking at, (home, mentions, direct search)
    self.timelines        Containe all timelines with statuses, all Timeline
                          objects
    '''

    def __init__(self):
        self.api        = tyrs.container['api']
        self.conf       = tyrs.container['conf']
        self.timelines  = tyrs.container['timelines']
        self.buffers    = tyrs.container['buffers']
        tyrs.container.add('interface', self)
        self.last_read_home = self.conf.load_last_read()
        self.api.set_interface()
        self.resize_event     = False
        self.regex_retweet     = re.compile('^RT @\w+:')
        self.refresh_token    = False
        self.buffer           = 'home'
        self.charset = sys.stdout.encoding
        # resize event
        signal.signal(signal.SIGWINCH, self.sigwinch_handler)
        self.init_screen()
        self.first_update()

    def init_screen(self):

        self.screen = curses.initscr()
        curses.noecho()         # Dont print anything
        #curses.cbreak()
        self.screen.timeout(500)
        self.screen.keypad(1)        # Use of arrow keys
        try:
            curses.curs_set(0)      # Dont display cursor
        except curses.error:
            pass
        curses.meta(1)          # allow 8bits inputs
        self.init_colors()
        self.maxyx = self.screen.getmaxyx()

        self.screen.refresh()

    def init_colors(self):
        curses.start_color()
        self.init_rgb_colors()
        self.init_color_pairs()

    def init_rgb_colors(self):
        if curses.can_change_color():
            for i in range(len(self.conf.color_set)):
                if not self.conf.color_set[i]:
                    continue
                else:
                    rgb = self.conf.color_set[i]
                    curses.init_color(i, rgb[0], rgb[1], rgb[2])
    
    def init_color_pairs(self):
        bgcolor = self.init_background()
        curses.init_pair(0, curses.COLOR_BLACK, bgcolor)    # 0 black
        curses.init_pair(1, curses.COLOR_RED, bgcolor)      # 1 red
        curses.init_pair(2, curses.COLOR_GREEN, bgcolor)    # 2 green
        curses.init_pair(3, curses.COLOR_YELLOW, bgcolor)   # 3 yellow
        curses.init_pair(4, curses.COLOR_BLUE, bgcolor)     # 4 blue
        curses.init_pair(5, curses.COLOR_MAGENTA, bgcolor)  # 5 magenta
        curses.init_pair(6, curses.COLOR_CYAN, bgcolor)     # 6 cyan
        curses.init_pair(7, curses.COLOR_WHITE, bgcolor)    # 7 white

    def init_background(self):
        bgcolor = False
        if self.conf.params['transparency']:
            curses.use_default_colors()
            bgcolor = -1
        return bgcolor

    def first_update(self):
        updates = ['home', 'direct', 'mentions', 'user_retweet', 'favorite']
        for buff in updates:
            self.api.update_timeline(buff)
            self.timelines[buff].reset()
            self.timelines[buff].all_read()
        self.display_timeline()

    def handle_resize_event(self):
        self.resize_event = False
        curses.endwin()
        self.set_max_window_size()
        self.display_redraw_screen()
        curses.doupdate()

    def change_buffer(self, buffer):
        self.screen.clear()
        self.buffer = buffer
        self.timelines[buffer].reset()
        self.display_timeline()
    
    def navigate_buffer(self, nav):
        '''Navigate with the arrow, mean nav should be -1 or +1'''
        index = self.buffers.index(self.buffer)
        new_index = index + nav
        if new_index >= 0 and new_index < len(self.buffers):
            self.change_buffer(self.buffers[new_index])

    def display_flash_message(self):
        if self.api.flash_message.event and not self.refresh_token:
            msg = self.api.flash_message.get_msg()
            level = self.api.flash_message.level
            msg_color = { 0: 'info_msg', 1: 'warning_msg', }
            self.screen.addstr(0, 3, msg.encode(self.charset), self.get_color(msg_color[level]))
            self.api.flash_message.reset()
            self.screen.refresh()

    def erase_flash_message(self):
        try:
            self.screen.addstr(0,3, '                               ')
        except curses.error:
            pass

    def display_update_msg(self):
        self.api.flash_message.event = 'update'
        self.display_flash_message()
    
    def display_redraw_screen(self):
        self.screen.erase()
        self.set_max_window_size()
        self.display_timeline()

    def set_max_window_size(self):
        self.maxyx = self.screen.getmaxyx()

    def display_timeline(self):
        '''Main entry to display a timeline, as it does not take arguments,
           make sure to set self.buffer before
        '''
        try:
            if not self.refresh_token:
                self.set_max_window_size()
                self.set_date()

                timeline = self.select_current_timeline()
                statuses_count = len(timeline.statuses)

                self.display_flash_message()
                self.display_activities()
                self.display_help_bar()

                # It might have no tweets yet, we try to retrieve some then
                if statuses_count  == 0:
                    self.api.update_timeline(self.buffer)
                    timeline.reset()

                self.current_y = 1
                for i in range(len(timeline.statuses)):
                    if i >= timeline.first:
                        self.check_for_last_read(timeline.statuses[i].id)
                        br = self.display_status(timeline.statuses[i], i)
                        if not br:
                            break
                timeline.unread = 0 
                if self.buffer == 'home':
                    self.conf.save_last_read(timeline.last_read)
                self.screen.refresh()
                self.check_current_not_on_screen()
        except curses.error:
            logging.error('Curses error for display_timeline')
            pass

    def check_for_last_read(self, id):
        if self.buffer == 'home':
            if self.last_read_home == str(id):
                self.screen.hline(self.current_y, 1, '-', self.maxyx[1]-3)
                self.current_y += 1

    def set_date(self):
        self.date = time.strftime("%d %b", time.gmtime())

    def select_current_timeline(self):
        return self.timelines[self.buffer]

    def check_current_not_on_screen(self):
        '''TODO this hack should be solved when we realy display tweets'''
        timeline = self.select_current_timeline()
        if timeline.current > timeline.last:
            timeline.current = timeline.last
            self.display_redraw_screen()
            self.display_timeline()

    def display_activities(self):
        '''Main entry to display the activities bar'''
        if self.conf.params['activities']:
            maxyx = self.screen.getmaxyx()
            max_x = maxyx[1]
            self.screen.addstr(0, max_x - 23, ' ')
            for b in self.buffers:
                self.display_buffer_activities(b)
                self.display_counter_activities(b)

    def display_buffer_activities(self, buff):
        display = { 
                'home': 'H', 'mentions': 'M', 'direct': 'D', 
                'search': 'S', 'user': 'U', 'favorite': 'F',
                'thread': 'T', 'user_retweet': 'R'}
        if self.buffer == buff:
            self.screen.addstr(display[buff], self.get_color('current_tab'))
        else:
            self.screen.addstr(display[buff], self.get_color('other_tab'))

    def display_counter_activities(self, buff):
        self.select_current_timeline().all_read()
        if buff in ['home', 'mentions', 'direct']:
            unread = self.timelines[buff].unread
            if unread == 0:
                color = 'read'
            else:
                color = 'unread'

            self.screen.addstr(':%s ' % str(unread), self.get_color(color))

    def display_help_bar(self):
        '''The help bar display at the bottom of the screen,
           for keysbinding reminder'''
        if self.conf.params['help']:
            maxyx = self.screen.getmaxyx()
            self.screen.addnstr(maxyx[0] -1, 2,
                'help:? up:%s down:%s tweet:%s retweet:%s reply:%s home:%s mentions:%s update:%s' %
                               (chr(self.conf.keys['up']),
                                chr(self.conf.keys['down']),
                                chr(self.conf.keys['tweet']),
                                chr(self.conf.keys['retweet']),
                                chr(self.conf.keys['reply']),
                                chr(self.conf.keys['home']),
                                chr(self.conf.keys['mentions']),
                                chr(self.conf.keys['update']),
                               ), maxyx[1] -4, self.get_color('text')
            )

    def display_status (self, status, i):
        ''' Display a status (tweet) from top to bottom of the screen,
        depending on self.current_y, an array [status, panel] is return and
        will be stock in a array, to retreve status information (like id)
        @param status, the status to display
        @param i, to know on witch status we're display (this could be refactored)
        @return True if the tweet as been displayed, to know it may carry on to display some
                more, otherwise return False
        '''

        timeline = self.select_current_timeline()
        self.is_retweet(status)

        # The content of the tweets is handle
        # text is needed for the height of a panel
        try:
            header  = self.get_header(status)
        except UnicodeDecodeError:
            header = 'encode error'

        # We get size and where to display the tweet
        size = self.get_size_status(status)
        length = size['length']
        height = size['height']
        start_y = self.current_y
        start_x = self.conf.params['margin']
        # We leave if no more space left
        if start_y + height +1 > self.maxyx[0]:
            return False

        panel = curses.newpad(height, length)

        if self.conf.params['tweet_border'] == 1:
            if self.conf.params['old_skool_border']:
                panel.border('|','|','-','-','+','+','+','+')
            else:
                if self.conf.params['compact']:
                    panel.border(curses.ACS_VLINE, curses.ACS_VLINE, curses.ACS_HLINE, ' ',
                                 curses.ACS_ULCORNER, curses.ACS_URCORNER, ' ', ' ')
                else:
                    panel.border(0)

        # Highlight the current status
        if timeline.current == i:
            panel.addstr(0,3, header, self.get_color('current_tweet'))
        else:
            panel.addstr(0, 3, header, self.get_color('header'))

        self.display_text(panel, status)
        try:
            panel.refresh(0, 0, start_y, start_x,
                start_y + height, start_x + length)
        except curses.error:
            pass
        # An adjustment to compress a little the display
        if self.conf.params['compact']:
            c = -1
        else:
            c = 0

        self.current_y = start_y + height + c
        timeline.last = i

        return True

    def get_text(self, status):
        text = html_unescape(status.text.replace('\n', ' '))
        if status.rt:
            text = text.split(':')[1:]
            text = ':'.join(text)

            if hasattr(status, 'retweeted_status'):
                if hasattr(status.retweeted_status, 'text') \
                        and len(status.retweeted_status.text) > 0:
                    text = status.retweeted_status.text
        return text

    def display_text(self, panel, status):
        '''needed to cut words properly, as it would cut it in a midle of a
        world without. handle highlighting of '#' and '@' tags.
        '''
        text = self.get_text(status)
        words = text.split(' ')
        margin = self.conf.params['margin']
        padding = self.conf.params['padding']
        myself = self.api.myself.screen_name
        curent_x = padding
        line = 1

        hashtag = encode('#')
        attag = encode('@')


        for word in words:
            word = encode(word)
            if curent_x + len(word) > self.maxyx[1] - (margin + padding)*2:
                line += 1
                curent_x = padding

            if word != '':
                # The word is an HASHTAG ? '#'
                if word[0] == hashtag:
                    panel.addstr(line, curent_x, word, self.get_color('hashtag'))
                # Or is it an 'AT TAG' ? '@'
                elif word[0] == attag:
                    # The AT TAG is,  @myself
                    if word == attag + myself or word == attag + myself+ encode(':'):
                        panel.addstr(line, curent_x, word, self.get_color('highlight'))
                    # @anyone
                    else:
                        panel.addstr(line, curent_x, word, self.get_color('attag'))
                # It's just a normal word
                else:
                    try:
                        panel.addstr(line, curent_x, word, self.get_color('text'))
                    except curses.error:
                        pass
                curent_x += len(word) + 1

                # We check for ugly empty spaces
                while panel.inch(line, curent_x -1) == ord(' ') and panel.inch(line, curent_x -2) == ord(' '):
                    curent_x -= 1

    def get_size_status(self, status):
        '''Allow to know how height will be the tweet, it calculate it exactly
           as it will display it.
        '''
        length = self.get_max_lenght()
        margin = self.conf.params['margin']
        padding = self.conf.params['padding']
        x = padding+margin
        y = 1
        txt = self.get_text(status)
        words = txt.split(' ')
        for w in words:
            if x+len(w) > length - (padding+margin)*2:
                y += 1
                x = padding+margin
            x += len(w)+1

        height = y + 2
        size = {'length': length, 'height': height}
        return size

    def get_max_lenght(self):
        adjust = self.conf.params['margin'] + self.conf.params['padding']
        return self.maxyx[1] - adjust

    def get_time(self, status):
        '''Handle the time format given by the api with something more
        readeable
        @param  date: full iso time format
        @return string: readeable time
        '''
        if self.conf.params['relative_time'] == 1 and self.buffer != 'direct':
            result =  status.GetRelativeCreatedAt()
        else:
            hour = time.gmtime(status.GetCreatedAtInSeconds() - time.altzone)
            result = time.strftime('%H:%M', hour)
            if time.strftime('%d %b', hour) != self.date:
                result += time.strftime(' - %d %b', hour)

        return result

    def get_header(self, status):
        retweeted = ''
        reply = ''
        retweet_count = ''
        retweeter = ''
        source = self.get_source(status)
        nick = self.get_nick(status)
        time = self.get_time(status)

        if self.is_reply(status):
            reply = u' \u2709'
        if status.rt:
            retweeted = u" \u267b "
            retweeter = nick
            nick = self.origin_of_retweet(status)

        if self.get_retweet_count(status):
            retweet_count = str(self.get_retweet_count(status))

        header_template = self.conf.params['header_template'] 
        header = unicode(header_template).format(
            time = time,
            nick = nick,
            reply = reply,
            retweeted = retweeted,
            source = source,
            retweet_count = retweet_count,
            retweeter = retweeter
            )

        return encode(header)

    def get_source(self, status):
        source = ''
        if hasattr(status, 'source'):
            source = get_source(status.source)

        return source

    def get_nick(self, status):
        if hasattr(status, 'user'):
            nick = status.user.screen_name
        else:
            #Used for direct messages
            nick = status.sender_screen_name

        return nick

    def get_retweet_count(self, status):
        if hasattr(status, 'retweet_count'):
            return status.retweet_count

    def is_retweet(self, status):
        status.rt = self.regex_retweet.match(status.text)
        return status.rt

    def is_reply(self, status):
        if hasattr(status, 'in_reply_to_screen_name'):
            reply = status.in_reply_to_screen_name
            if reply:
                return True
        return False

    def origin_of_retweet(self, status):
        '''When its a retweet, return the first person who tweet it,
           not the retweeter
        '''
        origin = status.text
        origin = origin[4:]
        origin = origin.split(':')[0]
        origin = str(origin)
        return origin

    def tear_down(self, *dummy):
        '''Last function call when quiting, restore some defaults params'''
        self.screen.keypad(0)
        curses.echo()
        curses.nocbreak()
        curses.curs_set(1)
        curses.endwin()

    def sigwinch_handler(self, *dummy):
        '''Resize event callback'''
        self.resize_event = True

    def clear_statuses(self):
        timeline = self.select_current_timeline()
        timeline.statuses = [timeline.statuses[0]]
        timeline.count_statuses()
        timeline.reset()

    def current_status(self):
        '''@return the status object itself'''
        timeline = self.select_current_timeline()
        return timeline.statuses[timeline.current]

    def get_color(self, color):
        '''Return the curses code, with bold if enable of the color
           given in argument of the function
           @return color_pair code
        '''
        cp = curses.color_pair(self.conf.colors[color]['c'])
        if self.conf.colors[color]['b']:
            cp |= curses.A_BOLD

        return cp

    def move_down(self):
        timeline = self.select_current_timeline()
        if timeline.current < timeline.count - 1:
            if timeline.current >= timeline.last:
                timeline.first += 1
            timeline.current += 1
        else:
            self.lazzy_load()

    def lazzy_load(self):
        timeline = self.select_current_timeline()
        timeline.page += 1
        statuses = self.api.retreive_statuses(self.buffer, timeline.page)
        timeline.append_old_statuses(statuses)
        timeline.first += 1
        timeline.current += 1

    def move_up(self):
        timeline = self.select_current_timeline()
        if timeline.current > 0:
            # if we need to move up the list to display
            if timeline.current == timeline.first:
                timeline.first -= 1
            timeline.current -= 1

    def back_on_bottom(self):
        timeline = self.select_current_timeline()
        timeline.current = timeline.last

    def back_on_top(self):
        timeline = self.select_current_timeline()
        timeline.current = 0
        timeline.reset()

    def openurl(self):
        urls = get_urls(self.current_status().text)
        for url in urls:
            try:
                os.system(self.conf.params['openurl_command'] % url)
            except:
                pass 

    def current_user_info(self):
        User(self.current_status().user)



# -*- coding: utf-8 -*-
# Copyright © 2011 Nicolas Paris <nicolas.caen@gmail.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import tyrs
import curses
import curses.ascii
from utils import encode, get_urls
try:
    from shorter.ur1ca import Ur1caUrlShorter
    from shorter.bitly import BitLyUrlShorter
    from shorter.msudpl import MsudplUrlShorter
    from shorter.custom import CustomUrlShorter
except ImportError:
    pass

try:
    from shorter.googl import GooglUrlShorter
except ImportError:
    pass

class Editor(object):

    def __init__(self, data=None):
        self.conf = tyrs.container['conf']
        self.interface = tyrs.container['interface'] 
        self.interface.refresh_token = True
        self.init_content(data)
        self.init_win()
        self.start_edit()
        self.tear_down()

    def init_content(self, data):
        self.content = ''
        if data:
            self.content = encode(data)


    def init_win(self):
        self.change_cursor(1)
        self.set_window_size()

        self.win = self.interface.screen.subwin(
                self.size['height'], self.size['width'],
                self.size['start_y'], self.size['start_x'])

        self.win.keypad(1)
        self.init_border()
        self.display_header()
        self.win.move(2,2)
        self.maxyx = self.win.getmaxyx()
        self.display_content()

    def init_border(self):
        if self.conf.params['old_skool_border']:
            self.win.border('|','|','-','-','+','+','+','+')
        else:
            self.win.border(0)

    def display_header(self):
        counter = self.count_chr()
        header = ' %s %s ' % (self.params['header'], str(counter))
        self.win.addstr(0, 3, encode(header), curses.color_pair(self.conf.colors['header']['c']))

    def set_window_size(self):
        maxyx = self.interface.screen.getmaxyx()

        # Set width
        if maxyx[1] > self.params['width']:
            width = self.params['width']
        else:
            width = maxyx[1] - 4 # 4: leave 2pix on every side at least

        # Set height
        height = int(self.params['char'] / width) + 4

        # Start of EditWin, display in the middle of the main screen
        if self.conf.params['box_position'] != 1:
            start_y = maxyx[0]/2 - int(height/2)
        else:
            start_y = maxyx[0] - height
        start_x = maxyx[1]/2 - int(width/2)
        self.sizeyx = (height, width)

        self.size = {
                'height': height, 'width': width,
                'start_y': start_y, 'start_x': start_x
        }


    def start_edit(self):

        while True:
            ch = self.win.getch()
            if ch == curses.KEY_UP or ch == curses.KEY_DOWN \
                    or ch == curses.KEY_LEFT or ch == curses.KEY_RIGHT:
                continue

            elif ch == 10:          # ENTER: send the tweet
                break

            elif ch == 27:        # ESC: abord
                self.content = None
                break

            elif ch == 127 or ch == curses.KEY_BACKSPACE:       # DEL
                if len(self.content) > 0:
                    self.content = self.content[:-1]
            elif curses.ascii.unctrl(ch) == '^U':
                self.shorter_url()
            else:
                self.content += chr(ch)

            self.refresh()

    def refresh(self):
        self.win.erase()
        self.init_win()
        self.win.refresh()

    def display_content(self):
        x = 2
        y = 2
        words = self.content.split(' ')
        max = self.win.getmaxyx()
        for w in words:
            if x+len(w) > max[1] - 4:
                y += 1
                x = 2
            self.win.addstr(y, x, w, self.interface.get_color('text'))
            x += len(w)+1

    def count_chr(self):
        i = 0
        token = False
        for ch in self.content:
            if not token:
                i += 1
                if not ord(ch) <= 128:
                    token = True
            else:
                token = False
        return i

    def change_cursor(self, opt):
        try:
            curses.curs_set(opt)
        except curses.error:
            pass

    def tear_down(self):
        self.win.erase()
        self.change_cursor(0)
        self.interface.refresh_token = False

class TweetEditor(Editor):
    params = {'char': 200, 'width': 80, 'header': _("What's up?")}

    def shorter_url(self):
        self._set_service()
        long_urls = get_urls(self.content)
        for long_url in long_urls:
            short_url = self.shorter.do_shorter(long_url)
            try:
                self.content = self.content.replace(long_url, short_url)
            except UnicodeDecodeError:
                pass

    def _set_service(self):
        service = self.conf.params['url_shorter']
        if service == 'bitly':
            self.shorter = BitLyUrlShorter() 
        elif service == 'googl':
            self.shorter = GooglUrlShorter()
        elif service == 'msudpl':
            self.shorter = MsudplUrlShorter()
        elif service == 'custom':
            self.shorter = CustomUrlShorter()
        else:
            self.shorter = Ur1caUrlShorter()


class NickEditor(Editor):
    params = {'char': 40, 'width': 40, 'header': _("Entry a name")}

class SearchEditor(Editor):
    params = {'char': 40, 'width': 40, 'header': _("Search for something?")}



# -*- coding: utf-8 -*-
# Copyright © 2011 Nicolas Paris <nicolas.caen@gmail.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
import os
import sys
import tyrs
from htmlentitydefs import entitydefs

def set_console_title():
    try:
        sys.stdout.write("\x1b]2;Tyrs\x07")
    except:
        pass

def cut_attag(name):
    if name[0] == '@':
        name = name[1:]
    return name

def encode(string):
    try:
        return string.encode(sys.stdout.encoding, 'replace')
    except AttributeError:
        return string

def html_unescape(str):
    """ Unescapes HTML entities """
    def entity_replacer(m):
        entity = m.group(1)
        if entity in entitydefs:
            return entitydefs[entity]
        else:
            return m.group(0)

    return re.sub(r'&([^;]+);', entity_replacer, str)


def get_source(source):
    if source != 'web':
        source = source.split('>')
        source = source[1:]
        source = ' '.join(source)
        source = source.split('<')[:1]
        source = source[:1]
        source = ' '.join(source)
    return source

def open_image(user):
    image = user.profile_image_url
    command = tyrs.container['conf'].params['open_image_command']
    os.system(command % image)


def get_urls(text):
    return re.findall('http[s]?://(?:[a-zA-Z]|[0-9]|[$-_@.&+]|[!*\(\),]|(?:%[0-9a-fA-F][0-9a-fA-F]))+', text)



# -*- coding: utf-8 -*-
# Copyright © 2011 Nicolas Paris <nicolas.caen@gmail.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from utils import encode

class FlashMessage(object):

    message = {
        'update': [
            _('Updating timeline...'),
            _('Couldn\'t retrieve tweets')
            ],
        'tweet': [
            _('Your tweet was sent'),
            _('Couldn\'t send tweet'),
            ],
        'retweet': [
            _('Your retweet was sent'),
            _('Couldn\'t send retweet'),
            ],
        'destroy': [
            _('You have deleted the tweet'),
            _('Couldn\'t delete tweet'),
            ],
        'favorite': [
            _('The tweet was added to favorites list'),
            _('Couldn\'t add tweet to favorites list'),
            ],
        'favorite_del': [
            _('Tweet was removed from favorites list'),
            _('Couldn\'t delete tweet on favorites list'),
            ],
        'direct': [
            _('Direct message was sent'),
            _('Couldn\'t send direct message'),
            ],
        'follow': [
            _('You are now following %s'),
            _('Couldn\'t follow %s')
            ],
        'unfollow': [
            _('You are not following %s anymore'),
            _('Couldn\'t stop following %s')
            ],
        'search': [
            _('Search results for %s'),
            _('Couldn\'t search for %s'),
            ],
        'empty': [
            '',''
        ]
        }

    def __init__(self):
        self.reset()

    def reset(self):
        self.level = 0
        self.event = None
        self.string = None

    def warning(self):
        self.level = 1

    def get_msg(self):
        return self.compose_msg()

    def compose_msg(self):
        msg = self.message[self.event][self.level]
        if self.string != None: 
            msg = self.message[self.event][self.level] % self.string
        
        return ' ' +msg+ ' '

def print_ask_service(token_file):
    print ''
    print encode(_('Couldn\'t find any profile.'))
    print ''
    print encode(_('It should reside in: %s')) % token_file
    print encode(_('If you want to setup a new account, then follow these steps'))
    print encode(_('If you want to skip this, just press return or ctrl-C.'))
    print ''

    print ''
    print encode(_('Which service do you want to use?'))
    print ''
    print '1. Twitter'
    print '2. Identi.ca'
    print ''

def print_ask_root_url():
    print ''
    print ''
    print encode(_('Which root url do you want? (leave blank for default, https://identi.ca/api)'))
    print ''



# -*- coding: utf-8 -*-
# Copyright © 2011 Nicolas Paris <nicolas.caen@gmail.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

class Container(object):
    '''
    Contain main classes that we need thought all the programm
    such as conf, api and ui
    '''
    _container = {}

    def __setitem__(self, key, value):
        self._container[key] = value

    def __getitem__(self, key):
        return self._container[key]

    def add(self, name, dependency):
        self[name] = dependency



# -*- coding: utf-8 -*-
# Copyright © 2011 Nicolas Paris <nicolas.caen@gmail.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import urllib2

class UrlShorter(object):

    def _quote_url(self, url):
        long_url = urllib2.quote(url)
        long_url = long_url.replace('/', '%2F')
        return long_url

    def _get_request(self, url, data=None):
        return urllib2.urlopen(url, data).read()



# -*- coding: utf-8 -*-
# Copyright © 2011 Nicolas Paris <nicolas.caen@gmail.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import urllib2
try:
    import json
except:
    import simplejson as json

from urlshorter import UrlShorter

APIKEY = 'apiKey=R_f806c2011339080ea0b623959bb8ecff'
VERION = 'version=2.0.1'
LOGIN  = 'login=tyrs'

class BitLyUrlShorter(UrlShorter):

    def __init__(self):
        self.base = 'http://api.bit.ly/shorten?%s&%s&%s&longUrl=%s'

    def do_shorter(self, url):
        long_url = self._quote_url(url)
        request = self.base % (VERION, LOGIN, APIKEY, long_url)
        response = json.loads(urllib2.urlopen(request).read())
        return response['results'][url]['shortUrl']



# -*- coding: utf-8 -*-
# Copyright © 2011 Nicolas Paris <nicolas.caen@gmail.com>
# Copyright © 2011 Natal Ngétal <hobbestig@cpan.org>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
import urllib
from urlshorter import UrlShorter

class MsudplUrlShorter(UrlShorter):
    def __init__(self):
        self.base = "http://msud.pl"
        self.pt   = re.compile('<p>Whouah ! This a very beautiful url :\) <a href="(.*?)">')
        self.pt_yet_in_base   = re.compile('and whouah! It\'s very beautiful <a href="(.*?)">')

    def do_shorter(self, longurl):
        values = {'submit' : 'Generate my sexy url', 'sexy_url': longurl}

        data = urllib.urlencode(values)
        resp = self._get_request(self.base, data)
        short = self.pt.findall(resp)
        if len(short) == 0:
            short = self.pt_yet_in_base.findall(resp)

        if len(short) > 0:
            return self.base + '/' + short[0]



# -*- coding: utf-8 -*-
# Copyright © 2011 Nicolas Paris <nicolas.caen@gmail.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import tyrs
import curses
from help import Help
from utils import open_image

class Keys(object):
    '''
    This class handle the main keysbinding, as the main method contain every
    keybinding, every case match a key to a method call, there is no logical
    here
    '''
    def __init__(self):
        self.conf       = tyrs.container['conf']
        self.interface  = tyrs.container['interface']
        self.api        = tyrs.container['api']

    def handleKeyBinding(self):
        '''Should have all keybinding handle here'''
        while True:

            if self.interface.resize_event:
                self.interface.handle_resize_event()
                self.interface.erase_flash_message()
                self.interface.display_timeline()

            ch = self.interface.screen.getch()

            # DOWN
            if ch == self.conf.keys['down'] or ch == curses.KEY_DOWN:
                self.interface.move_down()
            # UP
            elif ch == self.conf.keys['up'] or ch == curses.KEY_UP:
                self.interface.move_up()
            # LEFT
            elif ch == self.conf.keys['left'] or ch == curses.KEY_LEFT:
                self.interface.navigate_buffer(-1)
            # RIGHT
            elif ch == self.conf.keys['right'] or ch == curses.KEY_RIGHT:
                self.interface.navigate_buffer(+1)
            # TWEET
            elif ch == self.conf.keys['tweet']:
                self.api.tweet()
            # RETWEET
            elif ch == self.conf.keys['retweet']:
                self.api.retweet()
            # RETWEET AND EDIT
            elif ch == self.conf.keys['retweet_and_edit']:
                self.api.retweet_and_edit()
            # DELETE TwEET
            elif ch == self.conf.keys['delete']:
                self.api.destroy()
            # MENTIONS
            elif ch == self.conf.keys['mentions']:
                self.interface.change_buffer('mentions')
            # HOME TIMELINE
            elif ch == self.conf.keys['home']:
                self.interface.change_buffer('home')
            # CLEAR
            elif ch == self.conf.keys['clear']:
                self.interface.clear_statuses()
            # UPDATE
            elif ch == self.conf.keys['update']:
                self.api.update_timeline(self.interface.buffer)
            # FOLLOW SELECTED
            elif ch == self.conf.keys['follow_selected']:
                self.api.follow_selected()
            # UNFOLLOW SELECTED
            elif ch == self.conf.keys['unfollow_selected']:
                self.api.unfollow_selected()
            # FOLLOW
            elif ch == self.conf.keys['follow']:
                self.api.follow()
            # UNFOLLOW
            elif ch == self.conf.keys['unfollow']:
                self.api.unfollow()
            # OPENURL
            elif ch == self.conf.keys['openurl']:
                self.interface.openurl()
            # BACK ON TOP
            elif ch == self.conf.keys['back_on_top']:
                self.interface.back_on_top()
            # BACK ON BOTTOM
            elif ch == self.conf.keys['back_on_bottom']:
                self.interface.back_on_bottom()
            # REPLY
            elif ch == self.conf.keys['reply']:
                self.api.reply()
            # GET DIRECT MESSAGE
            elif ch == self.conf.keys['getDM']:
                self.interface.change_buffer('direct')
            # SEND DIRECT MESSAGE
            elif ch == self.conf.keys['sendDM']:
                self.api.direct_message()
            # SEARCH
            elif ch == self.conf.keys['search']:
                self.api.search()
            # SEARCH USER
            elif ch == self.conf.keys['search_user']:
                self.api.find_public_timeline()
            # SEARCH MYSELF
            elif ch == self.conf.keys['search_myself']:
                self.api.my_public_timeline()
            elif ch == self.conf.keys['search_current_user']:
                self.api.find_current_public_timeline()
            # Redraw screen
            elif ch == self.conf.keys['redraw']:
                self.interface.display_redraw_screen()
            # Help
            elif ch == ord('?'):
                Help()
            # Create favorite
            elif ch == self.conf.keys['fav']:
                self.api.set_favorite()
            # Get favorite
            elif ch == self.conf.keys['get_fav']:
                self.api.get_favorites()
            # Destroy favorite
            elif ch == self.conf.keys['delete_fav']:
                self.api.destroy_favorite()
            # Thread
            elif ch == self.conf.keys['thread']:
                self.api.get_thread()
            # Open image
            elif ch == self.conf.keys['open_image']:
                open_image(self.interface.current_status().user)
            # User info
            elif ch == ord('i'):
                self.interface.current_user_info()
            # QUIT
            elif ch == self.conf.keys['quit']:
                break
            else:
                continue

            self.interface.erase_flash_message()
            self.interface.display_timeline()



# -*- coding: utf-8 -*-
# Copyright © 2011 Nicolas Paris <nicolas.caen@gmail.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import tyrs
import logging
from urllib2 import URLError
from editor import *
from utils import cut_attag
from message import FlashMessage
from twitter import Api, TwitterError, Status

try:
    import json
except ImportError:
    import simplejson as json

class Tweets(object):
 
    def __init__(self):
        self.conf = tyrs.container['conf']
        self.timelines = tyrs.container['timelines']
        self.search_user = None
        self.search_word = None
        self.flash_message = FlashMessage()
    
    def set_interface(self):
        self.interface = tyrs.container['interface']

    def authentication(self):
        url = self.get_base_url()
        self.api = ApiPatch(
            self.conf.token[self.conf.service]['consumer_key'],
            self.conf.token[self.conf.service]['consumer_secret'],
            self.conf.oauth_token,
            self.conf.oauth_token_secret,
            base_url=url
        )
        self.set_myself()

    def get_base_url(self):
        url = None
        if self.conf.service == 'identica':
            url = self.conf.base_url

        return url

    def set_myself(self):
        self.myself = self.api.VerifyCredentials()
        self.conf.my_nick = self.myself.screen_name

    def tweet(self, data=None):
        tweet = TweetEditor(data).content
        if tweet:
            self.post_tweet(tweet)
        else:
            self.flash('empty')

    def post_tweet(self, tweet, reply_to=None):
        self.flash('tweet')
        try:
            return self.api.PostUpdate(tweet, reply_to)
        except TwitterError, e:
            self.error(e) 

    def retweet(self):
        self.flash('retweet')
        status = self.interface.current_status()
        try:
            self.api.PostRetweet(status.id)
        except TwitterError, e:
            self.error(e)

    def retweet_and_edit(self):
        status = self.interface.current_status()
        nick = status.user.screen_name
        data = 'RT @%s: %s' % (nick, status.text)
        self.tweet(data)

    def reply(self):
        status = self.interface.current_status()
        if hasattr(status, 'user'):
            nick = status.user.screen_name
        else:
            self.direct_message()
        data = '@' + nick + ' '
        tweet = TweetEditor(data).content
        if tweet:
            self.post_tweet(tweet, status.id)

    def destroy(self):
        self.flash('destroy')
        status = self.interface.current_status()
        try:
            self.api.DestroyStatus(status.id)
        except TwitterError, e:
            self.error(e)

    def direct_message(self):
        ''' Two editing box, one for the name, and one for the content'''
        nick = self.nick_for_direct_message()
        tweet = TweetEditor().content
        if tweet:
            self.send_direct_message(nick, tweet)

    def nick_for_direct_message(self):
        status = self.interface.current_status()
        if hasattr(status, 'user'):
            nick = status.user.screen_name
        else:
            nick = status.sender_screen_name
        nick = NickEditor(nick).content

        return nick

    def send_direct_message(self, nick, tweet):
        self.flash('direct')
        try:
            return self.api.PostDirectMessage(nick, tweet)
        except TwitterError, e:
            self.error(e)

    def follow(self):
        nick = NickEditor().content
        if nick:
            self.create_friendship(nick)

    def follow_selected(self):
        status = self.interface.current_status()
        if self.interface.is_retweet(status):
            nick = self.interface.origin_of_retweet(status)
        else:
            nick = status.user.screen_name
        self.create_friendship(nick)

    def unfollow(self):
        nick = NickEditor().content
        if nick:
            self.destroy_friendship(nick)       

    def unfollow_selected(self):
        nick = self.interface.current_status().user.screen_name
        self.destroy_friendship(nick)

    def create_friendship(self, nick):
        self.flash('follow', nick)
        try:
            self.api.CreateFriendship(nick)
        except TwitterError, e:
            self.error(e)

    def destroy_friendship(self, nick):
        self.flash('unfollow', nick)
        try:
            self.api.DestroyFriendship(nick)
        except TwitterError, e:
            self.error(e)

    def set_favorite(self):
        self.flash('favorite')
        status = self.interface.current_status()
        try:
            self.api.CreateFavorite(status)
        except TwitterError, e:
            self.error(e) 

    def destroy_favorite(self):
        self.flash('favorite_del')
        status = self.interface.current_status()
        try:
            self.api.DestroyFavorite(status)
        except TwitterError, e:
            self.error(e)

    def get_favorites(self):
        self.interface.change_buffer('favorite')

    def update_timeline(self, timeline):
        '''
        Retrieves tweets, don't display them
        @param the buffer to retreive tweets
        '''

        logging.debug('updating "{0}" timeline'.format(timeline))
        try:
            statuses = self.retreive_statuses(timeline)
            self.timelines[timeline].append_new_statuses(statuses)

        except TwitterError, e:
            self.update_error(e)

    def update_error(self, err):
        logging.error('Updating issue: {0}'.format(err))
        self.flash_message.event = 'update'
        self.flash_message.level = 1
        self.interface.display_flash_message()

    def retreive_statuses(self, timeline, page=None):
        self.interface.display_update_msg()
        if timeline == 'home':
            statuses = self.api.GetFriendsTimeline(retweets=True, page=page)
        elif timeline == 'mentions':
            statuses = self.api.GetMentions(page=page)
        elif timeline == 'user_retweet':
            statuses = self.api.GetUserRetweets()
        elif timeline == 'search' and self.search_word != '':
            statuses = self.api.GetSearch(self.search_word, page=page)
        elif timeline == 'direct':
            statuses = self.api.GetDirectMessages(page=page)
        elif timeline == 'user' and self.search_user != '':
            statuses = self.load_user_public_timeline(page=page)
        elif timeline == 'favorite':
            statuses = self.api.GetFavorites(page=page)
        elif timeline == 'thread':
            statuses = self.get_thread()
        self.interface.erase_flash_message()

        return statuses

    def find_public_timeline(self):
        nick = NickEditor().content
        if nick and nick != self.search_user:
            self.change_search_user(nick)
            self.load_user_public_timeline()
            self.interface.change_buffer('user')

    def find_current_public_timeline(self):
        self.change_search_user(self.interface.current_status().user.screen_name)
        self.load_user_public_timeline()
        self.interface.change_buffer('user')

    def change_search_user(self, nick):
        self.search_user = nick
        self.timelines['user'].empty()

    def my_public_timeline(self):
        self.change_search_user(self.myself.screen_name)
        self.load_user_public_timeline()

    def load_user_public_timeline(self, page=None):
        if self.search_user:
            return self.api.GetUserTimeline(self.search_user,
                    include_rts=True, page=page)
        else:
            return []

    def get_thread(self):
        try:
            status = self.interface.current_status()
            self.timelines['thread'].empty()
            self.statuses = [status]
            self.build_thread(status)
            self.timelines['thread'].append_new_statuses(self.statuses)
            self.interface.change_buffer('thread')
        except IndexError:
            return []

    def build_thread(self, status):
        if status.in_reply_to_status_id:
            try:
                reply_to = self.api.GetStatus(status.in_reply_to_status_id)
                self.statuses.append(reply_to)
                self.build_thread(reply_to)
            except TwitterError:
                pass

    def search(self):
        self.search_word = SearchEditor().content
        self.flash('search', self.search_word)
        self.timelines['search'].empty()
        try:
            self.timelines['search'].append_new_statuses(self.api.GetSearch(self.search_word))
            self.interface.change_buffer('search')
        except TwitterError, e:
            self.error(e)


    def flash(self, event, string=None):
        self.flash_message.event = event
        if string:
            self.flash_message.string = string
    
    def error(self, err=None):
        logging.warning('Error catch: {0}'.format(err))
        self.flash_message.warning()

class ApiPatch(Api):
    def PostRetweet(self, id):
        '''This code come from issue #130 on python-twitter tracker'''

        if not self._oauth_consumer:
            raise TwitterError("The twitter.Api instance must be authenticated.")
        try:
            if int(id) <= 0:
                raise TwitterError("'id' must be a positive number")
        except ValueError:
            raise TwitterError("'id' must be an integer")
        url = 'http://api.twitter.com/1/statuses/retweet/%s.json' % id
        json_data = self._FetchUrl(url, post_data={'dummy': None})
        data = json.loads(json_data)
        self._CheckForTwitterError(data)
        return Status.NewFromJsonDict(data)



# -*- coding: utf-8 -*-
# Copyright © 2011 Nicolas Paris <nicolas.caen@gmail.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
import tyrs
from utils import get_urls

class FilterStatus(object):

    def __init__(self):
        self.conf = tyrs.container['conf']

    def filter_status(self, status):
        self.setup_exception()
        try:
            if self.conf.filter['activate']:
                self.status = status
                if self.filter_without_url():
                    if self.filter_without_myself():
                        if self.filter_exception():
                            return True
            return False
        except:
            return False

    def filter_without_url(self):
        urls = get_urls(self.status.text)
        if len(urls) == 0:
            return True
        return False

    def filter_without_myself(self):
        if self.conf.filter['myself']:
            return True
        if self.conf.my_nick in self.status.text:
            return False
        else:
            return True


    def filter_exception(self):
        nick = self.status.user.screen_name
        if self.conf.filter['behavior'] == 'all':
            if not nick in self.exception:
                return True
        else:
            if nick in self.exception:
                return True
        return False

    def setup_exception(self):
        self.exception = self.conf.filter['except']
        self.exception.append(self.conf.my_nick)



# -*- coding: utf-8 -*-
# Copyright © 2011 Nicolas Paris <nicolas.caen@gmail.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# c: color value
# b: bold
colors = {
    'highlight':      {'c': 1},
    'header':         {'c': 6},
    'hashtag':        {'c': 3},
    'attag':          {'c': 2},
    'text':           {'c': 0},
    'warning_msg':    {'c': 1},
    'info_msg':       {'c': 2},
    'current_tweet':  {'c': 5},
    'current_tab':    {'c': 6},
    'other_tab':      {'c': 0},
    'unread':         {'c': 1},
    'read':           {'c': 0},
    'help':           {'c': 6},
}

color_set = [False, False, False, False, False, False, False, False]

key = {
    'up':                'k',
    'down':              'j',
    'left':              'J',
    'right':             'K',
    'quit':              'q',
    'tweet':             't',
    'clear':             'c',
    'retweet':           'r',
    'retweet_and_edit':  'R',
    'delete':            'C',
    'update':            'u',
    'follow_selected':   'f',
    'unfollow_selected': 'l',
    'follow':            'F',
    'unfollow':          'L',
    'openurl':           'o',
    'open_image':        '^I',
    'home':              'h',
    'mentions':          'm',
    'reply':             'M',
    'back_on_top':       'g',
    'back_on_bottom':    'G',
    'getDM':             'd',
    'sendDM':            'D',
    'search':            's',
    'search_user':       'U',
    'search_current_user': '^F',
    'search_myself':     '^U',
    'redraw':            '^L',
    'fav':               'b',
    'get_fav':           'B',
    'delete_fav':        '^B',
    'thread':            'T',
}

params = {
    'refresh':              2,
    'tweet_border':         1,
    'relative_time':        1,
    'retweet_by':           1,
    'margin':               1,
    'padding':              2,
    'openurl_command':      'firefox %s',
    'open_image_command':   'feh %s',
    'transparency':         True,
    'activities':           True,
    'compact':              False,
    'help':                 True,
    'old_skool_border':     False,
    'box_position':         1,
    'url_shorter':          'ur1ca',
    'logging_level':        3,
    'header_template':      ' {nick}{retweeted}{retweeter} - {time}{reply} {retweet_count} ',
}

filter = {
    'activate':         False,
    'myself':           False,
    'behavior':         'all',
    'except':           [],
}

token = {
    'twitter': {
        'consumer_key':     'Eq9KLjwH9sJNcpF4OOYNw',
        'consumer_secret':  '3JoHyvBp3L6hhJo4BJr6H5aFxLhSlR70ZYnM8jBCQ'
    },
    'identica': {
        'consumer_key':     '6b2cf8346a141d530739f72b977b7078',
        'consumer_secret':  '31b342b348502345d4a343a331e00edb'
    }
}



# -*- coding: utf-8 -*-
# Copyright © 2011 Nicolas Paris <nicolas.caen@gmail.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import sys
import httplib2
from urlshorter import UrlShorter

from apiclient.discovery import build

from oauth2client.file import Storage
from oauth2client.client import AccessTokenRefreshError
from oauth2client.client import OAuth2WebServerFlow
from oauth2client.tools import run

FLOW = OAuth2WebServerFlow(
    client_id='382344260739.apps.googleusercontent.com',
    client_secret='fJwAFxKWyW4rBmzzm6V3TVsZ',
    scope='https://www.googleapis.com/auth/urlshortener',
    user_agent='urlshortener-tyrs/1.0')

googl_token_file = os.environ['HOME'] + '/.config/tyrs/googl.tok'

class GooglUrlShorter(UrlShorter):

    def do_shorter(self, longurl):

        storage = Storage(googl_token_file)
        credentials = storage.get()
        if credentials is None or credentials.invalid:
            return 'need to register to use goog.gl'

        http = httplib2.Http()
        http = credentials.authorize(http)

        service = build("urlshortener", "v1", http=http)

        try:

            url = service.url()

            body = {"longUrl": longurl }
            resp = url.insert(body=body).execute()

            return resp['id']

        except AccessTokenRefreshError:
            pass

    def register_token(self):
        storage = Storage(googl_token_file)
        credentials = storage.get()
        if credentials is None or credentials.invalid:
            print 'There is no token file found for goo.gl'
            print 'A file will be generated for you'
            credentials = run(FLOW, storage)
